<?php

define('FPDF_FONTPATH', 'modules/pdfview/fpdf/font/');
require_once 'fpdf/fpdf.php';

class PDF extends FPDF
{
//variables of html parser
var $B = 0;
var $I = 0;
var $U;
var $HREF;
var $ALIGN='';
var $fontList;
var $issetfont;
var $issetcolor;

function PDF($orientation='P',$unit='mm',$format='A4')
{
  //Call parent constructor
  $this->FPDF($orientation,$unit,$format);
  //Initialization
  $this->B=0;
  $this->I=0;
  $this->U=0;
  $this->HREF='';
  $this->fontlist=array("arial","times","courier","helvetica","symbol");
  $this->issetfont = false;
  $this->issetcolor = false;
  $this->isblockquote = false;
}

function Footer()
{
  //Page footer
  $this->SetY(-15);
  $this->SetFont('Arial','I',8);
  $this->SetTextColor(128);
  $this->Cell(0,10,'Page '.$this->PageNo(),0,0,'C');
}

//////////////////////////////////////
//html parser

function WriteHTML($html)
{
  //remove all unsupported tags
  $html=strip_tags($html,"<b><u><i><a><img><p><br><strong><em><font><tr><blockquote>");
  $html=str_replace("\n",' ',$html); //replace carriage returns by spaces
  $a=preg_split('/<(.*)>/U',$html,-1,PREG_SPLIT_DELIM_CAPTURE); //explodes the string
  foreach($a as $i=>$e)
  {
    if($i%2==0)
    {
      // Print the text.
      if($this->HREF) {
        $this->PutLink($this->HREF,$e);
      }
      else if($this->ALIGN == 'center') {
        $this->Cell(0,5,$e,0,1,'C');
      }
      else if ($this->isblockquote == true) {
        $this->MultiCell(0, 5, $e, 0, 'J', 1);
      }
      else
        $this->Write(5,stripslashes(txtentities($e)));
      }
    else
    {
      //Tag
      if($e{0}=='/')
        $this->CloseTag(strtoupper(substr($e,1)));
      else
      {
        //Extract attributes
        $a2=explode(' ',$e);
        $tag=strtoupper(array_shift($a2));
        $attr=array();
        foreach($a2 as $v) {
          if(ereg('^([^=]*)=["\']?([^"\']*)["\']?$',$v,$a3)) {
            $attr[strtoupper($a3[1])]=$a3[2];
          }
        }
        $this->OpenTag($tag,$attr);
      }
    }
  }
}

function OpenTag($tag,$attr)
{
  //Opening tag
  switch($tag){
    case 'STRONG':
      $this->SetStyle('B',true);
      break;
    case 'EM':
      $this->SetStyle('I',true);
      break;
    case 'B':
    case 'I':
    case 'U':
      $this->SetStyle($tag,true);
      break;
    case 'A':
      $this->HREF=$attr['HREF'];
      break;
    case 'IMG':
      if(isset($attr['SRC']) and (isset($attr['WIDTH']) or isset($attr['HEIGHT']))) {
        if(!isset($attr['WIDTH'])) {
          $attr['WIDTH'] = 0;
        }
        if(!isset($attr['HEIGHT'])) {
          $attr['HEIGHT'] = 0;
        }
        $this->Image($attr['SRC'], $this->GetX(), $this->GetY(), px2mm($attr['WIDTH']), px2mm($attr['HEIGHT']));
      }
      break;
    case 'TR':
    case 'BR':
      $this->Ln(5);
      break;
    case 'P':
      $this->ALIGN=$attr['ALIGN'];
      break;
    case 'FONT':
      if (isset($attr['COLOR']) and $attr['COLOR']!='') {
        $coul=hex2dec($attr['COLOR']);
        $this->SetTextColor($coul['R'],$coul['G'],$coul['B']);
        $this->issetcolor=true;
      }
      if (isset($attr['FACE']) and in_array(strtolower($attr['FACE']), $this->fontlist)) {
        $this->SetFont(strtolower($attr['FACE']));
        $this->issetfont=true;
      }
      break;
    case 'BLOCKQUOTE':
      $this->Ln(10);
      $this->SetFillColor(238);
      $this->SetLeftMargin(25);
      $this->SetRightMargin(25);
      $this->isblockquote = true;
      break;
    case 'HR':
      if( $prop['WIDTH'] != '' ) {
        $Width = $prop['WIDTH'];
      }
      else {
        $Width = $this->w - $this->lMargin - $this->rMargin;
      }
      $this->Ln(2);
      $x = $this->GetX();
      $y = $this->GetY();
      $this->SetLineWidth(0.4);
      $this->Line($x,$y,$x+$Width,$y);
      $this->SetLineWidth(0.2);
      $this->Ln(2);
      break;
  }
}

function CloseTag($tag)
{
  //Closing tag
  switch($tag) {
    case 'STRONG':
      $tag='B';
    case 'EM':
      $tag='I';
    case 'B':
    case 'I':
    case 'U':
      $this->SetStyle($tag,false);
      break;
    case 'A':
      $this->HREF='';
      break;
    case 'FONT':
      if ($this->issetcolor) {
        $this->SetTextColor(0);
      }
      if ($this->issetfont) {
        $this->SetFont('arial');
        $this->issetfont=false;
      }
      break;
    case 'P':
      $this->ALIGN='';
      break;
    case 'BLOCKQUOTE':
      $this->SetFillColor(255);
      $this->SetMargins(1, 1, 1);
      $this->isblockquote = false;
      $this->Ln(10);
      break;
  }
}

function SetStyle($tag,$enable)
{
  //Modify style and select corresponding font
  $this->$tag+=($enable ? 1 : -1);
  $style='';
  foreach(array('B','I','U') as $s) {
    if($this->$s>0) {
      $style.=$s;
    }
  }
  $this->SetFont('',$style);
}

function PutLink($URL,$txt)
{
  //Put a hyperlink
  $this->SetTextColor(0,0,255);
  $this->SetStyle('U',true);
  $this->Write(5,$txt,$URL);
  $this->SetStyle('U',false);
  $this->SetTextColor(0);
}


}

//function hex2dec
//returns an associative array (keys: R,G,B) from
//a hex html code (e.g. #3FE5AA)
function hex2dec($couleur = "#000000"){
  $R = substr($couleur, 1, 2);
  $rouge = hexdec($R);
  $V = substr($couleur, 3, 2);
  $vert = hexdec($V);
  $B = substr($couleur, 5, 2);
  $bleu = hexdec($B);
  $tbl_couleur = array();
  $tbl_couleur['R']=$rouge;
  $tbl_couleur['G']=$vert;
  $tbl_couleur['B']=$bleu;
  return $tbl_couleur;
}

//conversion pixel -> millimeter in 72 dpi
function px2mm($px){
  return $px*25.4/72;
}

function txtentities($html){
  $trans = get_html_translation_table(HTML_ENTITIES);
  $trans = array_flip($trans);
  return strtr($html, $trans);
}

?>